classdef (Hidden) UnnamedElement < handle
    % UNNAMEDELEMENT handles unnamed elements in the meta-model
    
    % Copyright 2016-2017 The MathWorks, Inc.
    
    properties(Constant, Access=private)
        % example of a synthesized name of an unnamed element:
        % in a seq: 'temp-IncludedDataTypeSets(10)'
        % not in a seq: 'temp-IncludedDataTypeSets'
        %
        % The use of hyphen (-) is important to indicate that this is not a
        % valid short name of an element.
        EncodedNamePrefix =  'temp-';
    end
    
    methods(Static)
        % Determine if the passed element name is an unnamed element. This
        % also supports passing in the full path to the element.
        function isUnanmed = isUnnamed(elementName)
            isUnanmed = ~isempty(strfind(elementName,...
                autosar.api.UnnamedElement.EncodedNamePrefix));
        end
        
        % Given an unnamed element, return back the property name of
        % parent and index
        function [propertyName, indexOfElement] = decodeName(unnamedElement)
            if contains(unnamedElement, '(')
                s = regexp(unnamedElement,...
                    [autosar.api.UnnamedElement.EncodedNamePrefix...
                    '(?<propertyName>\w+)\((?<indexOfElement>\d+)\)'], 'names');
                indexOfElement = str2double(s.indexOfElement);
            else
                s = regexp(unnamedElement,...
                    [autosar.api.UnnamedElement.EncodedNamePrefix...
                    '(?<propertyName>\w+)'], 'names');
                indexOfElement = [];
            end
            propertyName = s.propertyName;
        end
        
        % return the full name of a given m3iObj. If the m3iObj contains
        % any unnamed elements in its path, then this method will use a
        % temporary name for that unnamed element comprising the property
        % name and index of its parent object.
        function elementPath = getQualifiedName(m3iObj)
            
            function str = nGetName(obj)
                name = obj.getOne('Name');
                if name.isvalid()
                    str = name.toString();
                else
                    % find property name of parent for m3iObj. If the parent is a
                    % sequence, find the index of m3iObj in this sequence
                    [propName,elementIndex] = autosar.api.UnnamedElement.getPropertyNameAndIndexOnParent(obj);
                    str = autosar.api.UnnamedElement.encodeName(propName, elementIndex);
                end
            end
            
            % loop backwards until we reach root
            elementPath = nGetName(m3iObj);
            rootModel = m3iObj.rootModel;
            parent = m3iObj.containerM3I;
            while parent.isvalid()
                if parent==rootModel
                    break
                end
                str = nGetName(parent);
                elementPath = sprintf('%s/%s', str, elementPath);
                parent = parent.containerM3I;
            end
            
            % strip leading AUTOSAR
            elementPath = regexprep(elementPath, '^AUTOSAR', '');
        end
    end
    
    methods (Static, Access = 'private')
        % find property name of parent for m3iObj. If the parent is a
        % sequence, find the index of m3iObj in this sequence
        function [parentPropName, index] = getPropertyNameAndIndexOnParent(m3iObj)
            index = -1; % assume element is not in a sequence
            
            parentPropertyAttr = Simulink.metamodel.arplatform.ModelFinder.getContainmentAttribute(m3iObj);
            parentPropName = parentPropertyAttr.name;
            isParentPropertySeq = ~strcmp(parentPropertyAttr.upper, '1');
            if isParentPropertySeq
                m3iParent = m3iObj.containerM3I;
                index = autosar.mm.Model.findObjectIndexInSequence(m3iParent.(parentPropName), m3iObj);
            end
        end
        
        % encode name of unnamed element by using property name on parent
        % and index
        function elementName = encodeName(propertyName, index)
            if (index == -1)
                % element is not in a sequence
                elementName = [autosar.api.UnnamedElement.EncodedNamePrefix propertyName];
            else
                if ~ischar(index)
                    index = num2str(index);
                end
                elementName = [autosar.api.UnnamedElement.EncodedNamePrefix...
                    propertyName '(' index ')'];
            end
        end
    end
end
